package statocaml_profile

  1. Overview
  2. Docs

Module Subsys.MakeSource

Parameters

module P : P

Signature

include P with type t = P.t
type t = P.t
val compare : t -> t -> int
val list : t list
val to_string : t -> string
val list_of_filename : string -> t list
val list_of_label : string -> t list
val to_dot_color : t -> string
Sourcemodule Map : Map.S with type key = t
Sourcemodule Set : Set.S with type elt = t
Sourceval of_commit : GH.Types.commit -> Set.t * int Map.t * int Map.t

From a commit, return the set of subsystems, let number of files for each subsystem and the number of lines added and deleted for each subsystem

Sourceval of_issue : GH.Types.issue -> Set.t
Sourceval commits_by_t : GH.Types.commit list -> GH.Types.commit list Map.t
Sourceval issues_by_t : GH.Types.issue list -> GH.Types.issue list Map.t