package statocaml_profile

  1. Overview
  2. Docs

Module S.Subs

include Subsys.P
type t
val compare : t -> t -> int
val list : t list
val to_string : t -> string
val list_of_filename : string -> t list
val list_of_label : string -> t list
val to_dot_color : t -> string
module Map : Map.S with type key = t
module Set : Set.S with type elt = t

From a commit, return the set of subsystems, let number of files for each subsystem and the number of lines added and deleted for each subsystem