Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
S.Subsval of_commit :
Statocaml_profile.Subsys.GH.Types.commit ->
Set.t * int Map.t * int Map.tFrom a commit, return the set of subsystems, let number of files for each subsystem and the number of lines added and deleted for each subsystem
val of_issue : Statocaml_profile.Subsys.GH.Types.issue -> Set.tval commits_by_t :
Statocaml_profile.Subsys.GH.Types.commit list ->
Statocaml_profile.Subsys.GH.Types.commit list Map.tval issues_by_t :
Statocaml_profile.Subsys.GH.Types.issue list ->
Statocaml_profile.Subsys.GH.Types.issue list Map.t